// Fichier I2C_Slave.c
/////////////////////////////////////////////////////////

/****************************************************************
 *                                                    			*
 * .Date de cration : 11/01/08                         		*
 * .Auteur : Guibert David                              		*       
 *           Cadic Matthieu										*
 *           Luzeau Yohan                                		*
 * .Descriptions :                                      		*
 *      - Librairie pour la gestion de l'I2C en mode esclave   	*
 *                                                      		*
 *                                                      		*
 *                                                      		*
 * .Remarques :                                         		*
 *      - Gestion des ordre reus par interruption plutt		*
 *		  que par polling								   		*
 *      - Adresse de la carte droite 0x4E          				*
 *                                                      		*
 ****************************************************************/



#include <i2c.h>	   		//gestion de la communication I2C
#include <p18f252.h>		// librairie du pic

#include "I2C_Slave.h"		// Header de la bibliothque




extern unsigned char 			AvG;								// Valeur du capteur avant gauche (mis  jour toutes les 20 ms)
extern unsigned char 			AvD;								// Valeur du capteur avant droit (mis  jour toutes les 20 ms)
extern unsigned char 			ArG;								// Valeur du capteur arrire gauche (mis  jour toutes les 20 ms)
extern unsigned char 			ArD;								// Valeur du capteur arrire droit (mis  jour toutes les 20 ms)





// -----------------------------------------------------------------------
/* Initialise le module I2C
 * Configure les ports entre
 * Configure le module en esclave
 * Spcifie l'adresse et configure les interruptions
*/
void I2C_Init(void)
{
	TRIS_SCL = 1;				// Configure les ports en entre
	TRIS_SDA = 1;				// Car on est en mode esclave

	OpenI2C(SLAVE_7, SLEW_OFF);	// configuration du pic en esclave sur 7 bits d'adresse (+ une lecture criture), vitesse sur 100 Khz
 
	SSPADD = SLAVE_ADRESS;		// Adresse de l'esclave
	SSPCON1bits.CKP = 1;  		// On relache l'horloge

	PIE1bits.SSPIE=1;      		// autorisation de l'interruption de l'i2c 
  	PIR1bits.SSPIF=0; 			// on dsactive le bit SSPIF pour 
								// permettre a l'interruption d'avoir lieu
}





/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//                            Fonctions		                               // 
//                                                                         // 
/////////////////////////////////////////////////////////////////////////////

// -----------------------------------------------------------------------
/* Fonction  appeler lors d'un interruption sur l'I2C
*/
unsigned char InterruptI2C (void)
{
	unsigned char 			Adresse=0;					// Dernire adresse reue (Utilis pour dterminer s'il s'agit d'une transmission ou d'une rception)
	static unsigned char 	NbData=0;					// Nombre de donnes transmises depuis le dbut de la trame
	unsigned char 			ReceivedValue=0;			// Mmorise la dernire valeur reue
	
	
	// Transmission : le matre demande des donnes
	// Rception	: le matre envoie des donnes

	// -------------------------- Adresse / Transmission ----------------------------
	if(!SSPSTATbits.D_A && SSPSTATbits.R_W)		// S'il s'agit d'une adresse en mode transmission
	{
		SSPBUF = AvG;							// On transmet la premire donne 
		NbData=1;								// On met  un le nombre de donnes transmises
		SSPCON1bits.CKP = 1;  					// On relache l'horloge
		return 0;
	}

	// -------------------------- Adresse / Rception ----------------------------
	if(!SSPSTATbits.D_A && !SSPSTATbits.R_W)	// S'il s'agit d'une adresse en mode rception
	{
		Adresse=SSPBUF;							// On lit la donne (l'adresse) pour vider le buffer		
		return 0;
	}
	
	// -------------------------- Donne / Transmission ----------------------------
	if(SSPSTATbits.D_A && SSPSTATbits.R_W)		// S'il s'agit d'une donne en mode transmission
	{
		switch (NbData)
		{
			case 0: 		SSPBUF = AvG;	break;		// On transmet la donne
			case 1: 		SSPBUF = AvD;	break;		// On transmet la donne
			case 2: 		SSPBUF = ArG;	break;		// On transmet la donne
			case 3: 		SSPBUF = ArD;	break;		// On transmet la donne
		}
		SSPCON1bits.CKP = 1;  							// On relache l'horloge
		if (NbData++>3) NbData=0;						// On incrmente le nombre de donnes transmises
		return 0;
	}
	
	// -------------------------- Donne / Rception ----------------------------
	if(SSPSTATbits.D_A && !SSPSTATbits.R_W)		// S'il s'agit d'une donne en mode rception
	{
		if (SSPSTATbits.BF)						// Vrifie qu'il y a une donne  lire 
		{
			ReceivedValue=SSPBUF;						// On lit la donne			
		}
	}	

	return ReceivedValue;	
}
