// Fichier Servos.c
/////////////////////////////////////////////////////////

/********************************************************
 *                                                      *
 *              En-tete du module de servo              *
 *                                                      *
 * .Date de cration : 19/12/07                         *
 * .Auteur : Jrmy Gardais                             *       
 *                                                      *
 * .Descriptions :                                      *
 *      - code des fonctions pour les servos            *
 *                                                      *
 * .Grandes modifications :                             *
 *  __________________________________________________  *
 * |   DATE   |   Auteur   |          Detail          | *
 * |--------------------------------------------------| *
 * | 19/12/07 | J.Gardais  | Cration et code de la   | *
 * |          |&P.Lucidarme| fonctions: Servo_Init    | *
 * |          |            | InterruptVectorLow       | * 
 * |          |            | ServiceInterruptLow      | *
 * |          |            | StateMachineServo        | *
 * |--------------------------------------------------| *
 * | 23/01/08 |M.Cardineau | -ajout de la config du   | *
 * |          |            |timer1 et des ports       | *
 * | 24/01/08 |            | -Servo_Depose()          | *
 * |          |            | -Servo_Tir()             | * 
 * |          |            | -Interuption_Servomoteur() *
 * |--------------------------------------------------| *
 * | 18/02/08 |M.Cardineau | -OuvrirBarriere()        | *
 * |          |            | -FermerBarriere()        | *
 * |--------------------------------------------------| *
 * |          |            |                          | *
 * |--------------------------------------------------| *
 * |__________|____________|__________________________| *
 *                                                      *
 *                                                      *
 * .Remarques :                                         *
 *      - Commenter!                                    *
 *      - La librairie utilise le timer1 pour commander *
 * les servomoteurs                                     *
 *      - 0 dans la commande d'un servomoteur			*
 *                                                      *
 ********************************************************/

//------INCLUSIONS-------//
#include <p18f252.h>		//librairie du PIC
#include <timers.h>			//librairies pour les timer du PIC
#include <delays.h>
#include <adc.h>			// Pour les convertisseur A/N

#include "Servos.h"
#include "CAN.h"


//-----------------------//

volatile unsigned char 			AvG=0;								// Valeur du capteur avant gauche (mis  jour toutes les 20 ms)
volatile unsigned char 			AvD=0;								// Valeur du capteur avant droit (mis  jour toutes les 20 ms)
volatile unsigned char 			ArG=0;								// Valeur du capteur arrire gauche (mis  jour toutes les 20 ms)
volatile unsigned char 			ArD=0;								// Valeur du capteur arrire droit (mis  jour toutes les 20 ms)




/***********************************Initialisation des servo***************************************/

void Servo_Init(void)
{
	/***** Dfinition des ports en sortie *****/
	TRIS_SERVO_EJECT_GAUCHE=0;		
	TRIS_SERVO_EJECT_DROIT=0; 	
	TRIS_SERVO_PANNEAU_GAUCHE=0; 	//on configure les ports de commande des servomoteurs en sortie 	
	TRIS_SERVO_PANNEAU_DROIT=0; 	

	OpenTimer1(	TIMER_INT_ON 	&	//Autorise les interruptions. Met le bit TMR1IE de PIE1  1.
				T1_16BIT_RW 	&   //Mode 16 bits
				T1_SOURCE_INT	&   //Horloge interne
				T1_PS_1_2		&	//Prescaler  1
				T1_OSC1EN_OFF	&   //Pas d'oscillateur
				T1_SYNC_EXT_OFF	); 	//Pas de synchro externe

	/***** Dfinitions des bits pour les interruptions *******/
	
	PIR1bits.TMR1IF = 0;			// Eteint le flag du timer 1. 0->dclenchement et interruption
	IPR1bits.TMR1IP=0;				// Timer 3 dclench sur priorit basse

}






/******Variable globales pour les pwm*******/  //A VERIFIER QUE ACCESSIBLE UNIQUEMENT AU .C
int PWM_Servo_Eject_Gauche=0;
int PWM_Servo_Eject_Droit=0;	
int PWM_Servo_Panneau_Gauche=0;
int PWM_Servo_Panneau_Droit=0;
/*********************************************/
/*
//permet de dposer une balle (sortie puis rentre du doigt)
void Servo_Depose(void)
{
	Servo_Eject_Commande(RC_DEPOSE_ACTIF);
	Delay10KTCYx(75);
	Servo_Eject_Commande(RC_DEPOSE_REPOS);
}

//permet de tirer une balle (sortie puis rentre du doigt)
void Servo_Tir(void)
{
	Servo_Canon_Commande(RC_CANON_ACTIF);
	Delay10KTCYx(50);
	Servo_Canon_Commande(RC_CANON_REPOS);	
}

//met le servo barriere en position basse (les balles peuvent passer)
void OuvrirBarriere(void)
{
	Servo_Barriere_Commande(SERVO_PANNEAU_GAUCHE_OUVERT);
}

//met le servo barriere en position haute (les balles sont bloques)
void FermerBarriere(void)
{
	Servo_Barriere_Commande(SERVO_PANNEAU_GAUCHE_FERMER);
}

*/



/*************************fonction de commande des servo******************************/

void Servo_Ejecteur_Gauche (int Cmd)
{
	if (Cmd!=PWM_Servo_Eject_Gauche)					// Ne fait la modif que si la valeur est diffrente pour viter d'interdir les interruption
	{
		INTCONbits.GIE=0;						// Interdit les interruptions pendant la copie
		PWM_Servo_Eject_Gauche = Cmd;					// Met la commande  jour
		INTCONbits.GIE=1;						// Rautorise les interruption
	}
}

void Servo_Ejecteur_Droit (int Cmd)
{
	if (Cmd!=PWM_Servo_Eject_Droit)					// Ne fait la modif que si la valeur est diffrente pour viter d'interdir les interruption
	{
		INTCONbits.GIE=0;						// Interdit les interruptions pendant la copie
		PWM_Servo_Eject_Droit = Cmd;					// Met la commande  jour
		INTCONbits.GIE=1;						// Rautorise les interruption	
	}
}

void Servo_Panneau_Gauche(int Cmd)
{
	if (Cmd!=PWM_Servo_Panneau_Gauche)				// Ne fait la modif que si la valeur est diffrente pour viter d'interdir les interruption
	{
		INTCONbits.GIE=0;						// Interdit les interruptions pendant la copie
		PWM_Servo_Panneau_Gauche = Cmd;				// Met la commande  jour
		INTCONbits.GIE=1;						// Rautorise les interruption	
	}
}

void Servo_Panneau_Droit(int Cmd)
{
	if (Cmd!=PWM_Servo_Panneau_Droit)					// Ne fait la modif que si la valeur est diffrente pour viter d'interdir les interruption
	{
		INTCONbits.GIE=0;						// Interdit les interruptions pendant la copie
		PWM_Servo_Panneau_Droit = Cmd;					// Met la commande  jour
		INTCONbits.GIE=1;						// Rautorise les interruption	
	}
}




/************************************ Vecteur d'interruption Bas ***********************************/

#pragma code InterruptVectorLow = 0x18   // Vecteur d'interruption
void InterruptVectorLow(void)  
{	
	_asm
		goto ServiceInterruptLow	//Saut  la routine d'interruption basse
	_endasm
}
#pragma code


/************************************ Interruption **********************************/
#pragma interruptlow ServiceInterruptLow
void ServiceInterruptLow(void)
{
	if(PIR1bits.TMR1IF) //Dbordement du timer1
	{
		PIR1bits.TMR1IF = 0;		//Acquitement Timer1
		StateMachineServo();    	//Machine  tat des servo
	}
}
#pragma code








/************************************Machine  tat des servo***************************************/
                                                                                              

//Gestion de la machine  tat des servomoteurs
void StateMachineServo()  
{
	int	ValCAN;
	
	static char state = 12;  //Etat de copie des consignes
	static int Copie_PWM_Servo_Eject_Gauche;
	static int Copie_PWM_Servo_Eject_Droit;
	static int Copie_PWM_Servo_Panneau_Gauche;
	static int Copie_PWM_Servo_Panneau_Droit;
/*
	state = 0	: servo eject : phase  1
	state = 1 	: servo eject : phase  1 variable
 	state = 2 	: servo eject : phase  0 variable

	state = 3	: servo canon : phase  1
	state = 4 	: servo canon : phase  1 variable
 	state = 5 	: servo canon : phase  0 variable

	state = 6	: servo gauche : phase  1
	state = 7 	: servo gauche : phase  1 variable
 	state = 8 	: servo gauche : phase  0 variable

	state = 9	: servo droite : phase  1
	state = 10 	: servo droite : phase  1 variable
 	state = 11 	: servo droite : phase  0 variable

	state = 12  : phase  vide de 10ms pour complter  20ms et copie des consignes
*/
	switch(state)
	{
		/*********** Servo Eject ************/
		//Phase 0
		case 0: WriteTimer1(63035);	//Rechargement du timer de 0,5ms
				SERVO_EJECT_GAUCHE=1;
				state = 1;
				break;
		//Phase 1
		case 1: WriteTimer1(65535-Copie_PWM_Servo_Eject_Gauche);	//Rechargement du timer de 0  2ms
				state = 2;
				break;
		//Phase 2
		case 2: WriteTimer1(55535+Copie_PWM_Servo_Eject_Gauche);	//Complment  2ms de la valeur prcdente
				SERVO_EJECT_GAUCHE=0;
				state = 3;
				break;

		/*********** Servo Canon ************/
		//Phase 3
		case 3: WriteTimer1(63035);	//Rechargement du timer de 0,5ms
				SERVO_EJECT_DROIT=1;
				state = 4;
				break;
		//Phase 4
		case 4: WriteTimer1(65535-Copie_PWM_Servo_Eject_Droit);	//Rechargement du timer de 0  2ms
				state = 5;
				break;
		//Phase 5
		case 5: WriteTimer1(55535+Copie_PWM_Servo_Eject_Droit);	//Complment  2ms de la valeur prcdente
				SERVO_EJECT_DROIT=0;
				state = 6;
				break;

		/*********** Servo Barriere************/
		//Phase 6
		case 6: WriteTimer1(63035);	//Rechargement du timer de 0,5ms
				SERVO_PANNEAU_GAUCHE=1;
				state = 7;
				break;
		//Phase 7
		case 7: WriteTimer1(65535-Copie_PWM_Servo_Panneau_Gauche);								// Rechargement du timer de 0  2ms
				state = 8;
				break;
		//Phase 8
		case 8: WriteTimer1(55535+Copie_PWM_Servo_Panneau_Gauche);								// Complment  2ms de la valeur prcdente
				SERVO_PANNEAU_GAUCHE=0;
				state = 9;				
				break;

		/*********** Servo Option ************/
		//Phase 9
		case 9: WriteTimer1(63035);																// Rechargement du timer de 0,5ms
				SERVO_PANNEAU_DROIT=1;
				state = 10;
				break;
		//Phase 10
		case 10:WriteTimer1(65535-Copie_PWM_Servo_Panneau_Droit);								// Rechargement du timer de 0  2ms 
				state = 11;
				break;
		//Phase 11
		case 11: WriteTimer1(55535+Copie_PWM_Servo_Panneau_Droit);								// Complment  2ms de la valeur prcdente
				SERVO_PANNEAU_DROIT=0;
				state = 12;
				break;


		case 12: WriteTimer1(/*15535 // 19500*/ 17700);													// Rechargement du timer de 10ms (/*valeur thorique*/valeur rele)
				Copie_PWM_Servo_Eject_Gauche=PWM_Servo_Eject_Gauche;
				Copie_PWM_Servo_Eject_Droit=PWM_Servo_Eject_Droit;
				Copie_PWM_Servo_Panneau_Gauche=PWM_Servo_Panneau_Gauche;
				Copie_PWM_Servo_Panneau_Droit=PWM_Servo_Panneau_Droit;				
				
				
				
																								// Mesure les capteur de distance
				
				ValCAN=CAN_Convert(ADC_CH2);													// Capteur avant gauche
				ValCAN=ValCAN>>1;																// Divise par 2
				if (ValCAN>255) ValCAN=255;														// Limite  255
				AvG=(unsigned char)ValCAN;														// Recopie la valeur

				ValCAN=CAN_Convert(ADC_CH3);													// Capteur avant droit
				ValCAN=ValCAN>>1;																// Divise par 2
				if (ValCAN>255) ValCAN=255;														// Limite  255
				AvD=(unsigned char)ValCAN;														// Recopie la valeur
				
				ValCAN=CAN_Convert(ADC_CH0);													// Capteur arrire gauche
				ValCAN=ValCAN>>1;																// Divise par 2
				if (ValCAN>255) ValCAN=255;														// Limite  255
				ArG=(unsigned char)ValCAN;														// Recopie la valeur
				
				ValCAN=CAN_Convert(ADC_CH1);													// Capteur arrire droit
				ValCAN=ValCAN>>1;																// Divise par 2
				if (ValCAN>255) ValCAN=255;														// Limite  255
				ArD=(unsigned char)ValCAN;														// Recopie la valeur

				state=0;
				break;
	}

}

