// Fichier main.c
/////////////////////////////////////////////////////////

/****************************************************************
 *                                                    			*
 * .Date de cration : 11/01/08                         		*
 * .Auteur : P. Lucidarme	                             		*
 * .Descriptions :                                      		*
 *      - Fichier principal pour la carte servos			   	*
 *                                                      		*
 *                                                      		*
 *                                                      		*
 * .Remarques :                                         		*
 *																*
 ****************************************************************/



// ------------ Directives de compilation

#pragma config OSC = HSPLL    								// Oscilateur  quartz 10 Mhz + PLL=40 MHZ
#pragma config WDT = OFF									// Pas de  watchdog
#pragma config LVP = OFF									// Low voltage programming disabled


// ------------ Inclusions 

#include <p18f252.h>										// Pour les E/S du PIC
#include <delays.h>											// Pour les tempos
#include <stdio.h>											// Pour le printf 
#include <adc.h>											// Pour les convertisseurs A/N

#include "serial.h"											// Pour la liaison srie
#include "servos.h"											// Pour les servomoteurs
#include "CAN.h"											// Pour les convertisseurs A/N
#include "I2C_Slave.h"										// Pour l'I2C



#define		LED_AMBRE			PORTCbits.RC0				// Led ambre
#define		TRIS_LED_AMBRE		TRISCbits.TRISC0			// Registre de sens de la led ambre

#define		LED_ROUGE			PORTCbits.RC1				// Led rouge
#define		TRIS_LED_ROUGE		TRISCbits.TRISC1			// Registre de sens de la led rouge




#define RC_BRAS_GAUCHE_REPOS		6700		// 6575 Augmenter pour approcher de la bute
#define RC_BRAS_GAUCHE_INTER		3200		// 2200 Augmenter pour relever le sevo lorsqu'il est actif
#define RC_BRAS_GAUCHE_ACTIF		2275		// 2200 Augmenter pour relever le sevo lorsqu'il est actif

#define RC_BRAS_DROIT_REPOS			2050		// Diminuer pour approcher de la bute
#define RC_BRAS_DROIT_INTER			4850		// Diminuer pour relever le sevo lorsqu'il est actif
#define RC_BRAS_DROIT_ACTIF			6300		// Diminuer pour relever le sevo lorsqu'il est actif



extern unsigned char 			AvG;								// Valeur du capteur avant gauche (mis  jour toutes les 20 ms)
extern unsigned char 			AvD;								// Valeur du capteur avant droit (mis  jour toutes les 20 ms)
extern unsigned char 			ArG;								// Valeur du capteur arrire gauche (mis  jour toutes les 20 ms)
extern unsigned char 			ArD;								// Valeur du capteur arrire droit (mis  jour toutes les 20 ms)

volatile unsigned char			Commande_Servo=0;							// consigne pour les servos, parmi :

#define CONS_NONE				0x00

#define BRASG_REPOS				0x25
#define BRASG_ACTIF				0x63

#define BRASD_REPOS				0xA0
#define BRASD_ACTIF				0xF5
		
#define BRASDG_INTER			0x8C



// --------------------------------------------------------------------------

// Prototype de la routine de service d'interuptions de priorit haute
void Interrupt_High_Priority(void);							// Utilise pour l'IC



void main(void)
{
	TRIS_LED_ROUGE=0;										// Led rouge en sortie
	TRIS_LED_AMBRE=0;										// Led ambre en sortie	
	LED_ROUGE=1;											// Eteinds la led rouge
	LED_AMBRE=1;											// Eteinds la led ambre
	
	I2C_Init();												// Initialise l'I2C
	CAN_Init();												// Initialise les convertisseurs A/N
	Serial_Init();											// Initialise la liaison srie
	Servo_Init();											// Dmarre les servomoteurs

	RCONbits.IPEN=1;										// Autorise les niveau de priorit sur les interuptions
	
	
	// Autorise les interruptions	
	INTCONbits.PEIE=1;										// Autorise les interruptions de basse priorit
	INTCONbits.GIE=1;										// Autorise les interruptions de haute priorit
	

	Servo_Ejecteur_Droit  (RC_BRAS_DROIT_REPOS);							// Place les servos dans leur position de repos
	Servo_Ejecteur_Gauche  (RC_BRAS_GAUCHE_REPOS);							// Place les servos dans leur position de repos
	
	printf ("Carte Servos \n");								// Petit message sur la liaison srie pour indiquer qu'on vient de booter
	
	
	
		
	while (1)
	{

		if( Commande_Servo!=CONS_NONE ){
			
			switch( Commande_Servo ){
				case BRASG_REPOS :
					Servo_Ejecteur_Gauche( RC_BRAS_GAUCHE_REPOS );			// Place le bras gauche en position lev (inactif)
					break;
					
				case BRASG_ACTIF :
					Servo_Ejecteur_Droit( RC_BRAS_DROIT_REPOS );			// Place le bras droit en position lev (inactif)
					Servo_Ejecteur_Gauche( RC_BRAS_GAUCHE_ACTIF );			// Place le bras gauche en position baiss (actif)
					break;
					
				case BRASD_REPOS :
					Servo_Ejecteur_Droit( RC_BRAS_DROIT_REPOS );			// Place le bras droit en position lev (inactif)
					break;
					
				case BRASD_ACTIF :
					Servo_Ejecteur_Gauche( RC_BRAS_GAUCHE_REPOS );			// Place le bras gauche en position lev (inactif)
					Servo_Ejecteur_Droit( RC_BRAS_DROIT_ACTIF );			// Place le bras droit en position baiss (actif)
					break;

				case BRASDG_INTER :
					Servo_Ejecteur_Droit ( RC_BRAS_DROIT_INTER );			// Place le bras droit en position baiss (actif)
					Servo_Ejecteur_Gauche( RC_BRAS_GAUCHE_INTER );			// Place le bras droit en position baiss (actif)
					break;
					
			}

			Commande_Servo = CONS_NONE;
		}
		
/*		LED_AMBRE=!LED_AMBRE;								// Pour le dbugage
		Delay10KTCYx(10);
		printf ("AvG=%d\t\tAvD=%d\t\tArG=%d\t\tArD=%d\n",AvG,AvD,ArG,ArD);
*/
	}
	
	
}	





/////////////////////////////////////////////////////////////////////////////
//                                                                         //
//                            Interruptions                                // 
//                                                                         // 
/////////////////////////////////////////////////////////////////////////////



//----------------------------------------------------------------------------
// Vecteur d'interruptions de priorit haute (utilis pour l'IC)
#pragma code highVector=0x08						// Vecteur d'interruption haut
void atInterruptHigh(void)
{
	_asm goto Interrupt_High_Priority _endasm		// On saute  la fonction MyHighInterrupt
}
#pragma code 										// Retour  la zone de code




//----------------------------------------------------------------------------
// Routine de service de l'interruption haute priorite
#pragma interrupt Interrupt_High_Priority 
void Interrupt_High_Priority(void)
{
	
	if (PIR1bits.SSPIF) 										// C'est le MSSP qui a lev l'interruption, c'est pour l'IC
	{	int ret;
		
		PIR1bits.SSPIF = 0; 									// On rautorise l'interruption		
		Commande_Servo = InterruptI2C();											// On appelle la routine I2C pour traiter la requete
		
	}	
}